package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.ColorRecipesExplorerApplication;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.SimdutException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityAlreadyExistsException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityNotFoundException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.ModelException;
import dev.fyloz.trial.colorrecipesexplorer.core.io.file.FileHandler;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.files.FilesService;
import dev.fyloz.trial.colorrecipesexplorer.dao.MaterialDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class MaterialService extends GenericService<Material, MaterialDao> {

    private MixQuantityService mixQuantityService;
    private FilesService filesService;

    @Autowired
    public MaterialService(MaterialDao materialDao, MixQuantityService mixQuantityService, FilesService filesService) {
        super(materialDao, Material.class);
        this.mixQuantityService = mixQuantityService;
        this.filesService = filesService;
    }

    public List<Material> getAllByMaterialType(MaterialType materialType) {
        if (materialType == null) return new ArrayList<>();

        return dao.findAllByMaterialType(materialType);
    }

    /**
     * Récupère tous les produits qui ne sont pas des types de mélange.
     *
     * @return Tous les produits qui ne sont pas des types de mélange
     */
    public List<Material> getAllNotMixType() {
        return getAll()
                .stream()
                .filter(m -> !m.isMixType())
                .collect(Collectors.toList());
    }

    public List<Material> getAllOrdered() {
        return getAll().stream()
                .sorted(Comparator.comparing(Material::getName))
                .collect(Collectors.toList());
    }

    /**
     * Récupère le produit correspondant à un nom.
     *
     * @param name Le nom du produit à récupérer
     * @return Le produit correspondant au nom
     */
    public Material getByName(String name) {
        Optional<Material> found = dao.findByName(name);
        if (found.isEmpty()) throw new EntityNotFoundException(type, ModelException.IdentifierType.NAME, name);

        return found.get();
    }

    public Material save(@NotNull Material material, MultipartFile file) {
        addSimdut(file, material);

        return super.save(material);
    }

    @Override
    public Material update(Material material) {
        Optional<Material> materialByCode = dao.findByName(material.getName());
        if (materialByCode.isPresent() && !material.getId().equals(materialByCode.get().getId()))
            throw new EntityAlreadyExistsException(type, ModelException.IdentifierType.NAME, material.getName());

        return super.update(material);
    }

    @Override
    public void delete(Material material) {
        removeSimdut(material);
        super.delete(material);
    }

    /**
     * Vérifie si un produit est lié à un ou plusieurs mélanges.
     *
     * @param material Le produit à vérifier.
     * @return Si le produit est lié à d'autres mélanges.
     */
    public boolean isLinkedToMixes(Material material) {
        return mixQuantityService.existsByMaterial(material);
    }

    @Deprecated(since = "1.3.0", forRemoval = true)
    public void deleteIfNotLinked(Material material) {
        if (!isLinkedToMixes(material)) {
            delete(material);
        }
    }

    /**
     * Récupère le chemin vers le fichier SIMDUT d'un produit.
     *
     * @param material Le produit
     * @return Le chemin vers le fichier SIMDUT du produit
     */
    private String getSimdutPath(Material material) {
        return String.format("%s/simdut/%s_%s.pdf",
                ColorRecipesExplorerApplication.UPLOAD_LOCATION,
                material.getId(),
                material.getName());
    }

    /**
     * Crée le fichier SIMDUT sur le disque et y transfert le contenu du MultipartFile passé en paramètre.
     * <p>
     * Cette méthode retournera true si le simdut est null ou si le transfert du fichier vers le disque a fonctionné.
     * Cette méthode retournera false si la création du fichier échoue ou si une erreur survient lors du transfert.
     *
     * @param simdut   Le contenu du fichier SIMDUT
     * @param material Le produit du SIMDUT
     */
    public void addSimdut(MultipartFile simdut, Material material) {
        if (filesService.writeMultiPartFile(simdut, getSimdutPath(material))) throw new SimdutException(material);
    }

    /**
     * Met à jour le fichier SIMDUT sur le disque.
     *
     * @param simdut   Le contenu du fichier SIMDUT mis à jour
     * @param material Le produit du SIMDUT
     */
    public void updateSimdut(MultipartFile simdut, Material material) {
        removeSimdut(material);
        addSimdut(simdut, material);
    }

    /**
     * Supprime le fichier SIMDUT pour un produit.
     * <p>
     * Cette méthode retournera true si le fichier SIMDUT du produit n'existe pas ou si la suppression est faîtes.
     *
     * @param material Le produit dont on veut supprimer le fichier SIMDUT
     */
    public void removeSimdut(Material material) {
        filesService.deleteFile(getSimdutPath(material));
    }

    public String getSimdutFileName(Material material) {
        return String.format("%s_%s", material.getId(), material.getName());
    }
}
