package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityAlreadyExistsException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.ModelException;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.model.*;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.MixFormDto;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.core.utils.MixBuilder;
import dev.fyloz.trial.colorrecipesexplorer.dao.MixDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class MixService extends GenericService<Mix, MixDao> {

    private MaterialService materialService;
    private MixQuantityService mixQuantityService;
    private MixTypeService mixTypeService;

    @Autowired
    public MixService(MixDao mixDao, MaterialService materialService, MixQuantityService mixQuantityService, MixTypeService mixTypeService) {
        super(mixDao, Mix.class);
        this.materialService = materialService;
        this.mixQuantityService = mixQuantityService;
        this.mixTypeService = mixTypeService;
    }

    /**
     * Récupère les produits disponibles pour un mélange.
     * Le mélange peut ne pas exister.
     *
     * @param recipe La recette dans laquelle se trouve le mélange
     * @param mixId  L'identifiant du mélange (-1 si le mélange n'existe pas)
     * @return Les produits disponibles pour ce mélange
     */
    public Collection<Material> getAvailableMaterialsForMixId(Recipe recipe, Long mixId) {
        return existsById(mixId) ? getAvailableMaterialsForMix(getById(mixId)) : getAvailableMaterialsForNewMix(recipe);
    }

    /**
     * Récupère les produits disponibles pour un mélange existant.
     *
     * @param mix Le mélange
     * @return Les produits disponibles pour ce mélange
     */
    public Collection<Material> getAvailableMaterialsForMix(Mix mix) {
        return getAvailableMaterialsForNewMix(mix.getRecipe())
                .stream()
                .filter(m -> !m.equals(mix.getMixType().getMaterial()))
                .collect(Collectors.toList());
    }

    /**
     * Récupère les produits disponibles pour un mélange inexistant.
     *
     * @param recipe La recette dans laquelle se trouve le mélange
     * @return Les produits disponibles pour le nouveau mélange
     */
    public Collection<Material> getAvailableMaterialsForNewMix(Recipe recipe) {
        Collection<MixType> recipeMixTypes = recipe.getMixTypes();

        return materialService
                .getAll()
                .stream()
                .filter(m -> !m.isMixType() || recipeMixTypes.contains(mixTypeService.getByMaterial(m)))
                .sorted(Comparator.comparing(Material::getName))
                .sorted(Comparator.comparing(m -> m.getMaterialType().getName()))
                .collect(Collectors.toList());
    }

    @Transactional
    public void create(MixFormDto formDto) {
        Mix mix = new MixBuilder(mixTypeService, materialService)
                .withDto(formDto)
                .build();

        if (mix.getRecipe().hasMixType(mix.getMixType()))
            throw new EntityAlreadyExistsException(type, ModelException.IdentifierType.OTHER, "mixType", mix.getMixType());

        save(mix);
    }

    @Transactional
    public void edit(Mix mix, MixFormDto formDto) {
        mix = new MixBuilder(mixTypeService, materialService)
                .withMix(mix)
                .withDto(formDto)
                .build();

        update(mix);
    }

    @Deprecated(since = "1.3.0", forRemoval = true)
    public void deleteMix(Mix mix) {
        mixQuantityService.deleteAll(mix.getMixQuantities());
        delete(mix);
    }

    @Override
    public void delete(Mix mix) {
        mixQuantityService.deleteAll(mix.getMixQuantities());

        super.delete(mix);
    }

    @Override
    public void deleteById(Long id) {
        delete(getById(id));
    }

    /**
     * Crée une liste de MixQuantity, pour la création d'un mélange
     *
     * @param mix        Le mélange associé aux MixQuantity
     * @param materials  Les matériaux
     * @param quantities Les quantités
     * @return La liste des MixQuantity créés.
     */
    public List<MixQuantity> createMixQuantities(Mix mix, List<Material> materials, List<Float> quantities) {
        List<MixQuantity> mixQuantities = new ArrayList<>();

        for (int i = 0; i < materials.size(); i++) {
            Material material = materials.get(i);
            float quantity = quantities.get(i);

            MixQuantity mixQuantity = new MixQuantity(mix, material, quantity);
            mixQuantities.add(mixQuantity);
        }

        return mixQuantities;
    }
}
