package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.model.*;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.RecipeEditorFormDto;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.RecipeExplorerFormDto;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.files.ImagesService;
import dev.fyloz.trial.colorrecipesexplorer.dao.RecipeDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.MultiValueMap;

import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class RecipeService extends GenericService<Recipe, RecipeDao> {

    private CompanyService companyService;
    private MixService mixService;
    private StepService stepService;
    private ImagesService imagesService;

    @Autowired
    public RecipeService(RecipeDao recipeDao, CompanyService companyService, MixService mixService, StepService stepService, ImagesService imagesService) {
        super(recipeDao, Recipe.class);
        this.companyService = companyService;
        this.mixService = mixService;
        this.stepService = stepService;
        this.imagesService = imagesService;
    }

    /**
     * Récupère la liste des recettes associées à une compagnie.
     *
     * @param company La compagnie.
     * @return La liste des recettes associées à cette compagnie.
     */
    public List<Recipe> getByCompany(Company company) {
        return dao.findAllByCompany(company);
    }

    /**
     * Récupère une liste triée des mélanges pour une recette.
     *
     * @param recipe La recette dont on veut récupérer les mélanges
     * @return Une liste triée des mélanges.
     */
    public List<Mix> getSortedMixes(Recipe recipe) {
        List<Mix> mixes = recipe.getMixes();
        mixes.sort(Comparator.comparing(Mix::getId));
        return new ArrayList<>(mixes);  // Convertit le PersistentBag en ArrayList
    }

    /**
     * Récupère toutes les recettes dans la base de données et les classes par compagnie.
     *
     * @return Un Map contenant les recettes classées par compagnie.
     */
    public Map<Company, List<Recipe>> getRecipesByCompany() {
        return mappedByCompany(getAll());
    }

    /**
     * Met à jour une recette ainsi que ses étapes.
     *
     * @param recipeDto Les informations de la recette à mettre à jour
     * @return La recette mise à jour
     */
    public Recipe updateRecipeAndSteps(RecipeEditorFormDto recipeDto) {
        Recipe recipe = recipeDto.getRecipe();

        List<RecipeStep> steps = stepService.createAllForRecipe(recipe, recipeDto.getStep());
        return setSteps(recipe, steps);
    }

    /**
     * Met à jour les informations d'une recette trouvées dans l'explorateur de recette.
     *
     * @param form Le formulaire contenant les données mises à jour
     */
    @Transactional
    public void updateRecipeExplorerInfos(RecipeExplorerFormDto form) {
        long recipeId = form.getRecipeId();
        Map<Long, String> locations = form.getLocations();
        String note = form.getNote();

        Recipe recipe = getById(recipeId);

        // Note
        recipe.setNote(note);
        save(recipe);

        // Casiers
        for (Map.Entry<Long, String> location : locations.entrySet()) {
            Mix mix = mixService.getById(location.getKey());
            mix.setLocation(location.getValue());
            mixService.update(mix);
        }
    }

    @Override
    public void delete(Recipe recipe) {
        super.delete(recipe);
        removeAllFiles(recipe);
    }

    /**
     * Récupère le nom des images liées à une recette.
     *
     * @param recipe La recette dont on veut récupérer les images
     * @return Une liste contenant le nom des images liées à la recette.
     */
    public List<String> getImageFiles(Recipe recipe) {
        String imageName = getImageFileName(recipe);
        File[] allImages = imagesService.getDirectoryFile().listFiles((d, n) -> n.startsWith(imageName));

        if (allImages == null) return new ArrayList<>();
        return Arrays.stream(allImages).map(File::getName).collect(Collectors.toList());
    }

    // TODO test
    public int getNextImageIndex(Recipe recipe) {
        String imageName = getImageFileName(recipe);
        List<String> allImages = getImageFiles(recipe);
        List<Integer> indexes = allImages.stream().map(i -> Integer.parseInt(i.replace(imageName + "-", "").replace(".jpeg", ""))).collect(Collectors.toList());
        return indexes.size() > 0 ? Collections.max(indexes) + 1 : 0;
    }

    /**
     * Récupère les mélanges d'une recette triés selon leur identifiant.
     *
     * @param recipe La recette
     * @return Les mélanges de la recette triés selon leur identifiant
     */
    public Collection<Mix> getMixesSortedById(Recipe recipe) {
        return getMixesSorted(recipe, Comparator.comparing(Mix::getId));
    }

    /**
     * Récupère les mélanges d'une recette triés selon un comparateur.
     *
     * @param recipe     La recette
     * @param comparator Le comparateur, ou la méthode de triage
     * @return Les mélanges de la recette triés selon le comparateur
     */
    public Collection<Mix> getMixesSorted(Recipe recipe, Comparator<? super Mix> comparator) {
        return recipe
                .getMixes()
                .parallelStream()
                .sorted(comparator)
                .collect(Collectors.toList());
    }

    private Map<Company, List<Recipe>> mappedByCompany(List<Recipe> recipes) {
        List<Company> companies = companyService.getAll();
        Map<Company, List<Recipe>> mappedRecipes = new HashMap<>();

        for (Company c : companies) {
            List<Recipe> recipesForCompany = recipes.stream().filter(r -> r.getCompany().equals(c)).collect(Collectors.toList());

            if (recipesForCompany.size() > 0) {
                mappedRecipes.put(c, recipesForCompany);
            }
        }

        return mappedRecipes;
    }

    @Transactional
    protected Recipe setSteps(Recipe recipe, List<RecipeStep> steps) {
        stepService.deleteAll(recipe.getRecipeSteps());
        recipe.setRecipeSteps(steps);

        return update(recipe);
    }

    private void removeAllFiles(Recipe recipe) {
        getImageFiles(recipe).forEach(f -> imagesService.deleteImage(f));
    }

    public String getImageFileName(Recipe recipe) {
        return String.format("%s_%s", recipe.getId(), recipe.getName());
    }

    public String getImageFileNameWithIndex(Recipe recipe) {
        return getImageFileNameWithIndex(recipe, getNextImageIndex(recipe));
    }

    public String getImageFileNameWithIndex(Recipe recipe, int index) {
        return String.format("%s-%s", getImageFileName(recipe), index);
    }
}
